package com.muyu.web.mqtt;

import com.muyu.web.utils.ConversionUtil;
import com.muyu.vehicle.MessageData;
import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.admin.NewTopic;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaAdmin;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 回执消息类 MessageCallbackService
 *
 * @author Yangle
 * Date 2024/6/6 15:08
 */
@Service
@Log4j2
public class MessageCallbackService implements MqttCallback {

    @Autowired
    private KafkaTemplate kafkaTemplate;
    @Autowired
    private KafkaAdmin kafkaAdmin;
    @Override
    public void connectionLost(Throwable cause) {
        System.out.println("connectionLost:"+cause.getMessage());
    }


    @Override
    public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception {

        log.info("topic:{}",topic);
        log.info("Qos:{}",mqttMessage.getQos());
        log.info("message content:{}",new String(mqttMessage.getPayload()));
        String s = new String(mqttMessage.getPayload());
        MessageData main = ConversionUtil.main(s);
        try {
            List<NewTopic> newTopicList = createNewTopics(topic, main.getVin(),main);
            for (NewTopic newTopic : newTopicList) {
                kafkaAdmin.createOrModifyTopics(newTopic);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private List<NewTopic> createNewTopics(String topic, String vin, MessageData messageData) {
        List<NewTopic> newTopics = new ArrayList<>();
        String topicName = topic + "-" + vin;
        newTopics.add(new NewTopic(topicName, 8, (short) 1));
        ProducerRecord<String, Object> stringObjectProducerRecord = new ProducerRecord<>(topicName, messageData.toString());
        kafkaTemplate.send(stringObjectProducerRecord);
        return newTopics;
    }
    @Override
    public void deliveryComplete(IMqttDeliveryToken token) {
        System.out.println("deliveryComplete---------" + token.isComplete());
    }
}
